"""
Most of this file is taken from the Django project, which is BSD licensed.
"""

from distutils.core import setup
from distutils.command.install_data import install_data
from distutils.command.install import INSTALL_SCHEMES
import os
import sys

from mopidy import get_version

class osx_install_data(install_data):
    # On MacOS, the platform-specific lib dir is
    # /System/Library/Framework/Python/.../ which is wrong. Python 2.5 supplied
    # with MacOS 10.5 has an Apple-specific fix for this in
    # distutils.command.install_data#306. It fixes install_lib but not
    # install_data, which is why we roll our own install_data class.

    def finalize_options(self):
        # By the time finalize_options is called, install.install_lib is set to
        # the fixed directory, so we set the installdir to install_lib. The
        # install_data class uses ('install_data', 'install_dir') instead.
        self.set_undefined_options('install', ('install_lib', 'install_dir'))
        install_data.finalize_options(self)

if sys.platform == "darwin":
    cmdclasses = {'install_data': osx_install_data}
else:
    cmdclasses = {'install_data': install_data}

def fullsplit(path, result=None):
    """
    Split a pathname into components (the opposite of os.path.join) in a
    platform-neutral way.
    """
    if result is None:
        result = []
    head, tail = os.path.split(path)
    if head == '':
        return [tail] + result
    if head == path:
        return result
    return fullsplit(head, [tail] + result)

# Tell distutils to put the data_files in platform-specific installation
# locations. See here for an explanation:
# http://groups.google.com/group/comp.lang.python/browse_thread/
# thread/35ec7b2fed36eaec/2105ee4d9e8042cb
for scheme in INSTALL_SCHEMES.values():
    scheme['data'] = scheme['purelib']

# Compile the list of packages available, because distutils doesn't have
# an easy way to do this.
packages, data_files = [], []
root_dir = os.path.dirname(__file__)
if root_dir != '':
    os.chdir(root_dir)
project_dir = 'mopidy'

for dirpath, dirnames, filenames in os.walk(project_dir):
    # Ignore dirnames that start with '.'
    for i, dirname in enumerate(dirnames):
        if dirname.startswith('.'):
            del dirnames[i]
    if '__init__.py' in filenames:
        packages.append('.'.join(fullsplit(dirpath)))
    elif filenames:
        data_files.append([dirpath,
            [os.path.join(dirpath, f) for f in filenames]])

setup(
    name='Mopidy',
    version=get_version(),
    author='Stein Magnus Jodal',
    author_email='stein.magnus@jodal.no',
    packages=packages,
    package_data={'mopidy': ['backends/spotify/spotify_appkey.key']},
    cmdclass=cmdclasses,
    data_files=data_files,
    scripts=['bin/mopidy', 'bin/mopidy-scan'],
    url='http://www.mopidy.com/',
    license='Apache License, Version 2.0',
    description='MPD server with Spotify support',
    long_description=open('README.rst').read(),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: No Input/Output (Daemon)',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Multimedia :: Sound/Audio :: Players',
    ],
)



from copy import copy
import logging

logger = logging.getLogger('mopidy.backends.base')

class StoredPlaylistsController(object):
    """
    :param backend: backend the controller is a part of
    :type backend: :class:`mopidy.backends.base.Backend`
    :param provider: provider the controller should use
    :type provider: instance of :class:`BaseStoredPlaylistsProvider`
    """

    pykka_traversable = True

    def __init__(self, backend, provider):
        self.backend = backend
        self.provider = provider

    @property
    def playlists(self):
        """
        Currently stored playlists.

        Read/write. List of :class:`mopidy.models.Playlist`.
        """
        return self.provider.playlists

    @playlists.setter
    def playlists(self, playlists):
        self.provider.playlists = playlists

    def create(self, name):
        """
        Create a new playlist.

        :param name: name of the new playlist
        :type name: string
        :rtype: :class:`mopidy.models.Playlist`
        """
        return self.provider.create(name)

    def delete(self, playlist):
        """
        Delete playlist.

        :param playlist: the playlist to delete
        :type playlist: :class:`mopidy.models.Playlist`
        """
        return self.provider.delete(playlist)

    def get(self, **criteria):
        """
        Get playlist by given criterias from the set of stored playlists.

        Raises :exc:`LookupError` if a unique match is not found.

        Examples::

            get(name='a')            # Returns track with name 'a'
            get(uri='xyz')           # Returns track with URI 'xyz'
            get(name='a', uri='xyz') # Returns track with name 'a' and URI
                                     # 'xyz'

        :param criteria: one or more criteria to match by
        :type criteria: dict
        :rtype: :class:`mopidy.models.Playlist`
        """
        matches = self.playlists
        for (key, value) in criteria.iteritems():
            matches = filter(lambda p: getattr(p, key) == value, matches)
        if len(matches) == 1:
            return matches[0]
        criteria_string = ', '.join(
            ['%s=%s' % (k, v) for (k, v) in criteria.iteritems()])
        if len(matches) == 0:
            raise LookupError('"%s" match no playlists' % criteria_string)
        else:
            raise LookupError('"%s" match multiple playlists' % criteria_string)

    def lookup(self, uri):
        """
        Lookup playlist with given URI in both the set of stored playlists and
        in any other playlist sources.

        :param uri: playlist URI
        :type uri: string
        :rtype: :class:`mopidy.models.Playlist`
        """
        return self.provider.lookup(uri)

    def refresh(self):
        """
        Refresh the stored playlists in
        :attr:`mopidy.backends.base.StoredPlaylistsController.playlists`.
        """
        return self.provider.refresh()

    def rename(self, playlist, new_name):
        """
        Rename playlist.

        :param playlist: the playlist
        :type playlist: :class:`mopidy.models.Playlist`
        :param new_name: the new name
        :type new_name: string
        """
        return self.provider.rename(playlist, new_name)

    def save(self, playlist):
        """
        Save the playlist to the set of stored playlists.

        :param playlist: the playlist
        :type playlist: :class:`mopidy.models.Playlist`
        """
        return self.provider.save(playlist)


class BaseStoredPlaylistsProvider(object):
    """
    :param backend: backend the controller is a part of
    :type backend: :class:`mopidy.backends.base.Backend`
    """

    pykka_traversable = True

    def __init__(self, backend):
        self.backend = backend
        self._playlists = []

    @property
    def playlists(self):
        """
        Currently stored playlists.

        Read/write. List of :class:`mopidy.models.Playlist`.
        """
        return copy(self._playlists)

    @playlists.setter
    def playlists(self, playlists):
        self._playlists = playlists

    def create(self, name):
        """
        See :meth:`mopidy.backends.base.StoredPlaylistsController.create`.

        *MUST be implemented by subclass.*
        """
        raise NotImplementedError

    def delete(self, playlist):
        """
        See :meth:`mopidy.backends.base.StoredPlaylistsController.delete`.

        *MUST be implemented by subclass.*
        """
        raise NotImplementedError

    def lookup(self, uri):
        """
        See :meth:`mopidy.backends.base.StoredPlaylistsController.lookup`.

        *MUST be implemented by subclass.*
        """
        raise NotImplementedError

    def refresh(self):
        """
        See :meth:`mopidy.backends.base.StoredPlaylistsController.refresh`.

        *MUST be implemented by subclass.*
        """
        raise NotImplementedError

    def rename(self, playlist, new_name):
        """
        See :meth:`mopidy.backends.base.StoredPlaylistsController.rename`.

        *MUST be implemented by subclass.*
        """
        raise NotImplementedError

    def save(self, playlist):
        """
        See :meth:`mopidy.backends.base.StoredPlaylistsController.save`.

        *MUST be implemented by subclass.*
        """
        raise NotImplementedError



# encoding: utf-8

import os
import shutil
import sys
import tempfile

from mopidy.utils.path import (get_or_create_folder, mtime,
    path_to_uri, uri_to_path, split_path, find_files)

from tests import unittest, path_to_data_dir


class GetOrCreateFolderTest(unittest.TestCase):
    def setUp(self):
        self.parent = tempfile.mkdtemp()

    def tearDown(self):
        if os.path.isdir(self.parent):
            shutil.rmtree(self.parent)

    def test_creating_folder(self):
        folder = os.path.join(self.parent, 'test')
        self.assert_(not os.path.exists(folder))
        self.assert_(not os.path.isdir(folder))
        created = get_or_create_folder(folder)
        self.assert_(os.path.exists(folder))
        self.assert_(os.path.isdir(folder))
        self.assertEqual(created, folder)

    def test_creating_nested_folders(self):
        level2_folder = os.path.join(self.parent, 'test')
        level3_folder = os.path.join(self.parent, 'test', 'test')
        self.assert_(not os.path.exists(level2_folder))
        self.assert_(not os.path.isdir(level2_folder))
        self.assert_(not os.path.exists(level3_folder))
        self.assert_(not os.path.isdir(level3_folder))
        created = get_or_create_folder(level3_folder)
        self.assert_(os.path.exists(level2_folder))
        self.assert_(os.path.isdir(level2_folder))
        self.assert_(os.path.exists(level3_folder))
        self.assert_(os.path.isdir(level3_folder))
        self.assertEqual(created, level3_folder)

    def test_creating_existing_folder(self):
        created = get_or_create_folder(self.parent)
        self.assert_(os.path.exists(self.parent))
        self.assert_(os.path.isdir(self.parent))
        self.assertEqual(created, self.parent)

    def test_create_folder_with_name_of_existing_file_throws_oserror(self):
        conflicting_file = os.path.join(self.parent, 'test')
        open(conflicting_file, 'w').close()
        folder = os.path.join(self.parent, 'test')
        self.assertRaises(OSError, get_or_create_folder, folder)


class PathToFileURITest(unittest.TestCase):
    def test_simple_path(self):
        if sys.platform == 'win32':
            result = path_to_uri(u'C:/WINDOWS/clock.avi')
            self.assertEqual(result, 'file:///C://WINDOWS/clock.avi')
        else:
            result = path_to_uri(u'/etc/fstab')
            self.assertEqual(result, 'file:///etc/fstab')

    def test_folder_and_path(self):
        if sys.platform == 'win32':
            result = path_to_uri(u'C:/WINDOWS/', u'clock.avi')
            self.assertEqual(result, 'file:///C://WINDOWS/clock.avi')
        else:
            result = path_to_uri(u'/etc', u'fstab')
            self.assertEqual(result, u'file:///etc/fstab')

    def test_space_in_path(self):
        if sys.platform == 'win32':
            result = path_to_uri(u'C:/test this')
            self.assertEqual(result, 'file:///C://test%20this')
        else:
            result = path_to_uri(u'/tmp/test this')
            self.assertEqual(result, u'file:///tmp/test%20this')

    def test_unicode_in_path(self):
        if sys.platform == 'win32':
            result = path_to_uri(u'C:/æøå')
            self.assertEqual(result, 'file:///C://%C3%A6%C3%B8%C3%A5')
        else:
            result = path_to_uri(u'/tmp/æøå')
            self.assertEqual(result, u'file:///tmp/%C3%A6%C3%B8%C3%A5')


class UriToPathTest(unittest.TestCase):
    def test_simple_uri(self):
        if sys.platform == 'win32':
            result = uri_to_path('file:///C://WINDOWS/clock.avi')
            self.assertEqual(result, u'C:/WINDOWS/clock.avi')
        else:
            result = uri_to_path('file:///etc/fstab')
            self.assertEqual(result, u'/etc/fstab')

    def test_space_in_uri(self):
        if sys.platform == 'win32':
            result = uri_to_path('file:///C://test%20this')
            self.assertEqual(result, u'C:/test this')
        else:
            result = uri_to_path(u'file:///tmp/test%20this')
            self.assertEqual(result, u'/tmp/test this')

    def test_unicode_in_uri(self):
        if sys.platform == 'win32':
            result = uri_to_path( 'file:///C://%C3%A6%C3%B8%C3%A5')
            self.assertEqual(result, u'C:/æøå')
        else:
            result = uri_to_path(u'file:///tmp/%C3%A6%C3%B8%C3%A5')
            self.assertEqual(result, u'/tmp/æøå')


class SplitPathTest(unittest.TestCase):
    def test_empty_path(self):
        self.assertEqual([], split_path(''))

    def test_single_folder(self):
        self.assertEqual(['foo'], split_path('foo'))

    def test_folders(self):
        self.assertEqual(['foo', 'bar', 'baz'], split_path('foo/bar/baz'))

    def test_folders(self):
        self.assertEqual(['foo', 'bar', 'baz'], split_path('foo/bar/baz'))

    def test_initial_slash_is_ignored(self):
        self.assertEqual(['foo', 'bar', 'baz'], split_path('/foo/bar/baz'))

    def test_only_slash(self):
        self.assertEqual([], split_path('/'))


class FindFilesTest(unittest.TestCase):
    def find(self, path):
        return list(find_files(path_to_data_dir(path)))

    def test_basic_folder(self):
        self.assert_(self.find(''))

    def test_nonexistant_folder(self):
        self.assertEqual(self.find('does-not-exist'), [])

    def test_file(self):
        files = self.find('blank.mp3')
        self.assertEqual(len(files), 1)
        self.assert_(files[0], path_to_data_dir('blank.mp3'))

    def test_names_are_unicode(self):
        is_unicode = lambda f: isinstance(f, unicode)
        for name in self.find(''):
            self.assert_(is_unicode(name),
                '%s is not unicode object' % repr(name))


class MtimeTest(unittest.TestCase):
    def tearDown(self):
        mtime.undo_fake()

    def test_mtime_of_current_dir(self):
        mtime_dir = int(os.stat('.').st_mtime)
        self.assertEqual(mtime_dir, mtime('.'))

    def test_fake_time_is_returned(self):
        mtime.set_fake_time(123456)
        self.assertEqual(mtime('.'), 123456)



from mopidy import MopidyException

class MpdAckError(MopidyException):
    """See fields on this class for available MPD error codes"""

    ACK_ERROR_NOT_LIST = 1
    ACK_ERROR_ARG = 2
    ACK_ERROR_PASSWORD = 3
    ACK_ERROR_PERMISSION = 4
    ACK_ERROR_UNKNOWN = 5
    ACK_ERROR_NO_EXIST = 50
    ACK_ERROR_PLAYLIST_MAX = 51
    ACK_ERROR_SYSTEM = 52
    ACK_ERROR_PLAYLIST_LOAD = 53
    ACK_ERROR_UPDATE_ALREADY = 54
    ACK_ERROR_PLAYER_SYNC = 55
    ACK_ERROR_EXIST = 56

    error_code = 0

    def __init__(self, message=u'', index=0, command=u''):
        super(MpdAckError, self).__init__(message, index, command)
        self.message = message
        self.index = index
        self.command = command

    def get_mpd_ack(self):
        """
        MPD error code format::

            ACK [%(error_code)i@%(index)i] {%(command)s} description
        """
        return u'ACK [%i@%i] {%s} %s' % (
            self.__class__.error_code, self.index, self.command, self.message)

class MpdArgError(MpdAckError):
    error_code = MpdAckError.ACK_ERROR_ARG

class MpdPasswordError(MpdAckError):
    error_code = MpdAckError.ACK_ERROR_PASSWORD

class MpdPermissionError(MpdAckError):
    error_code = MpdAckError.ACK_ERROR_PERMISSION

    def __init__(self, *args, **kwargs):
        super(MpdPermissionError, self).__init__(*args, **kwargs)
        self.message = u'you don\'t have permission for "%s"' % self.command

class MpdUnknownCommand(MpdAckError):
    error_code = MpdAckError.ACK_ERROR_UNKNOWN

    def __init__(self, *args, **kwargs):
        super(MpdUnknownCommand, self).__init__(*args, **kwargs)
        self.message = u'unknown command "%s"' % self.command
        self.command = u''

class MpdNoExistError(MpdAckError):
    error_code = MpdAckError.ACK_ERROR_NO_EXIST

class MpdSystemError(MpdAckError):
    error_code = MpdAckError.ACK_ERROR_SYSTEM

class MpdNotImplemented(MpdAckError):
    error_code = 0

    def __init__(self, *args, **kwargs):
        super(MpdNotImplemented, self).__init__(*args, **kwargs)
        self.message = u'Not implemented'



from mopidy.backends import dummy as backend
from mopidy.frontends.mpd import dispatcher
from mopidy.frontends.mpd.protocol import status
from mopidy.mixers import dummy as mixer
from mopidy.models import Track

from tests import unittest

PAUSED = backend.PlaybackController.PAUSED
PLAYING = backend.PlaybackController.PLAYING
STOPPED = backend.PlaybackController.STOPPED

# FIXME migrate to using protocol.BaseTestCase instead of status.stats
# directly?


class StatusHandlerTest(unittest.TestCase):
    def setUp(self):
        self.backend = backend.DummyBackend.start().proxy()
        self.mixer = mixer.DummyMixer.start().proxy()
        self.dispatcher = dispatcher.MpdDispatcher()
        self.context = self.dispatcher.context

    def tearDown(self):
        self.backend.stop().get()
        self.mixer.stop().get()

    def test_stats_method(self):
        result = status.stats(self.context)
        self.assert_('artists' in result)
        self.assert_(int(result['artists']) >= 0)
        self.assert_('albums' in result)
        self.assert_(int(result['albums']) >= 0)
        self.assert_('songs' in result)
        self.assert_(int(result['songs']) >= 0)
        self.assert_('uptime' in result)
        self.assert_(int(result['uptime']) >= 0)
        self.assert_('db_playtime' in result)
        self.assert_(int(result['db_playtime']) >= 0)
        self.assert_('db_update' in result)
        self.assert_(int(result['db_update']) >= 0)
        self.assert_('playtime' in result)
        self.assert_(int(result['playtime']) >= 0)

    def test_status_method_contains_volume_which_defaults_to_0(self):
        result = dict(status.status(self.context))
        self.assert_('volume' in result)
        self.assertEqual(int(result['volume']), 0)

    def test_status_method_contains_volume(self):
        self.mixer.volume = 17
        result = dict(status.status(self.context))
        self.assert_('volume' in result)
        self.assertEqual(int(result['volume']), 17)

    def test_status_method_contains_repeat_is_0(self):
        result = dict(status.status(self.context))
        self.assert_('repeat' in result)
        self.assertEqual(int(result['repeat']), 0)

    def test_status_method_contains_repeat_is_1(self):
        self.backend.playback.repeat = 1
        result = dict(status.status(self.context))
        self.assert_('repeat' in result)
        self.assertEqual(int(result['repeat']), 1)

    def test_status_method_contains_random_is_0(self):
        result = dict(status.status(self.context))
        self.assert_('random' in result)
        self.assertEqual(int(result['random']), 0)

    def test_status_method_contains_random_is_1(self):
        self.backend.playback.random = 1
        result = dict(status.status(self.context))
        self.assert_('random' in result)
        self.assertEqual(int(result['random']), 1)

    def test_status_method_contains_single(self):
        result = dict(status.status(self.context))
        self.assert_('single' in result)
        self.assert_(int(result['single']) in (0, 1))

    def test_status_method_contains_consume_is_0(self):
        result = dict(status.status(self.context))
        self.assert_('consume' in result)
        self.assertEqual(int(result['consume']), 0)

    def test_status_method_contains_consume_is_1(self):
        self.backend.playback.consume = 1
        result = dict(status.status(self.context))
        self.assert_('consume' in result)
        self.assertEqual(int(result['consume']), 1)

    def test_status_method_contains_playlist(self):
        result = dict(status.status(self.context))
        self.assert_('playlist' in result)
        self.assert_(int(result['playlist']) in xrange(0, 2**31 - 1))

    def test_status_method_contains_playlistlength(self):
        result = dict(status.status(self.context))
        self.assert_('playlistlength' in result)
        self.assert_(int(result['playlistlength']) >= 0)

    def test_status_method_contains_xfade(self):
        result = dict(status.status(self.context))
        self.assert_('xfade' in result)
        self.assert_(int(result['xfade']) >= 0)

    def test_status_method_contains_state_is_play(self):
        self.backend.playback.state = PLAYING
        result = dict(status.status(self.context))
        self.assert_('state' in result)
        self.assertEqual(result['state'], 'play')

    def test_status_method_contains_state_is_stop(self):
        self.backend.playback.state = STOPPED
        result = dict(status.status(self.context))
        self.assert_('state' in result)
        self.assertEqual(result['state'], 'stop')

    def test_status_method_contains_state_is_pause(self):
        self.backend.playback.state = PLAYING
        self.backend.playback.state = PAUSED
        result = dict(status.status(self.context))
        self.assert_('state' in result)
        self.assertEqual(result['state'], 'pause')

    def test_status_method_when_playlist_loaded_contains_song(self):
        self.backend.current_playlist.append([Track()])
        self.backend.playback.play()
        result = dict(status.status(self.context))
        self.assert_('song' in result)
        self.assert_(int(result['song']) >= 0)

    def test_status_method_when_playlist_loaded_contains_cpid_as_songid(self):
        self.backend.current_playlist.append([Track()])
        self.backend.playback.play()
        result = dict(status.status(self.context))
        self.assert_('songid' in result)
        self.assertEqual(int(result['songid']), 0)

    def test_status_method_when_playing_contains_time_with_no_length(self):
        self.backend.current_playlist.append([Track(length=None)])
        self.backend.playback.play()
        result = dict(status.status(self.context))
        self.assert_('time' in result)
        (position, total) = result['time'].split(':')
        position = int(position)
        total = int(total)
        self.assert_(position <= total)

    def test_status_method_when_playing_contains_time_with_length(self):
        self.backend.current_playlist.append([Track(length=10000)])
        self.backend.playback.play()
        result = dict(status.status(self.context))
        self.assert_('time' in result)
        (position, total) = result['time'].split(':')
        position = int(position)
        total = int(total)
        self.assert_(position <= total)

    def test_status_method_when_playing_contains_elapsed(self):
        self.backend.playback.state = PAUSED
        self.backend.playback.play_time_accumulated = 59123
        result = dict(status.status(self.context))
        self.assert_('elapsed' in result)
        self.assertEqual(result['elapsed'], '59.123')

    def test_status_method_when_starting_playing_contains_elapsed_zero(self):
        self.backend.playback.state = PAUSED
        self.backend.playback.play_time_accumulated = 123 # Less than 1000ms
        result = dict(status.status(self.context))
        self.assert_('elapsed' in result)
        self.assertEqual(result['elapsed'], '0.123')

    def test_status_method_when_playing_contains_bitrate(self):
        self.backend.current_playlist.append([Track(bitrate=320)])
        self.backend.playback.play()
        result = dict(status.status(self.context))
        self.assert_('bitrate' in result)
        self.assertEqual(int(result['bitrate']), 320)



from mopidy import settings
from mopidy.frontends.mpd.protocol import handle_request
from mopidy.frontends.mpd.exceptions import (MpdPasswordError,
    MpdPermissionError)

@handle_request(r'^close$', auth_required=False)
def close(context):
    """
    *musicpd.org, connection section:*

        ``close``

        Closes the connection to MPD.
    """
    context.session.close()

@handle_request(r'^kill$')
def kill(context):
    """
    *musicpd.org, connection section:*

        ``kill``

        Kills MPD.
    """
    raise MpdPermissionError(command=u'kill')

@handle_request(r'^password "(?P<password>[^"]+)"$', auth_required=False)
def password_(context, password):
    """
    *musicpd.org, connection section:*

        ``password {PASSWORD}``

        This is used for authentication with the server. ``PASSWORD`` is
        simply the plaintext password.
    """
    if password == settings.MPD_SERVER_PASSWORD:
        context.dispatcher.authenticated = True
    else:
        raise MpdPasswordError(u'incorrect password', command=u'password')

@handle_request(r'^ping$', auth_required=False)
def ping(context):
    """
    *musicpd.org, connection section:*

        ``ping``

        Does nothing but return ``OK``.
    """
    pass



import logging
import os
import sys

logger = logging.getLogger('mopidy.utils')

def flatten(the_list):
    result = []
    for element in the_list:
        if isinstance(element, list):
            result.extend(flatten(element))
        else:
            result.append(element)
    return result

def import_module(name):
    __import__(name)
    return sys.modules[name]

def get_class(name):
    logger.debug('Loading: %s', name)
    if '.' not in name:
        raise ImportError("Couldn't load: %s" % name)
    module_name = name[:name.rindex('.')]
    class_name = name[name.rindex('.') + 1:]
    try:
        module = import_module(module_name)
        class_object = getattr(module, class_name)
    except (ImportError, AttributeError):
        raise ImportError("Couldn't load: %s" % name)
    return class_object



import logging

from spotify import Link, SpotifyError

from mopidy.backends.base import BasePlaybackProvider

logger = logging.getLogger('mopidy.backends.spotify.playback')

class SpotifyPlaybackProvider(BasePlaybackProvider):
    def pause(self):
        return self.backend.gstreamer.pause_playback()

    def play(self, track):
        if self.backend.playback.state == self.backend.playback.PLAYING:
            self.backend.spotify.session.play(0)
        if track.uri is None:
            return False
        try:
            self.backend.spotify.session.load(
                Link.from_string(track.uri).as_track())
            self.backend.spotify.session.play(1)
            self.backend.gstreamer.prepare_change()
            self.backend.gstreamer.set_uri('appsrc://')
            self.backend.gstreamer.start_playback()
            self.backend.gstreamer.set_metadata(track)
            return True
        except SpotifyError as e:
            logger.info('Playback of %s failed: %s', track.uri, e)
            return False

    def resume(self):
        return self.seek(self.backend.playback.time_position)

    def seek(self, time_position):
        self.backend.gstreamer.prepare_change()
        self.backend.spotify.session.seek(time_position)
        self.backend.gstreamer.start_playback()
        return True

    def stop(self):
        result = self.backend.gstreamer.stop_playback()
        self.backend.spotify.session.play(0)
        return result



def populate_playlist(func):
    def wrapper(self):
        for track in self.tracks:
            self.backend.current_playlist.add(track)
        return func(self)

    wrapper.__name__ = func.__name__
    wrapper.__doc__ = func.__doc__
    return wrapper
